/*
 * decimate - secure file deletion
 *
 * http://www.afro-productions.com
 *
 * Brought to you by the Afro Productions Frenzied Bear Division
 *
 * the algorithm is simple:
 *
 *  -  change access/mod times for file
 *  -  open the file for reading/writing.
 *  -  read the file and replace characters with random ones.
 *  -  close the file.
 *  -  sync the disks.
 *  -  do those steps x amount of times to insure it works. (should work first time, but oh well).
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <unistd.h>
#include <utime.h>

#define ROUND	0x04

#define S_VERBOSE (1 << 1)
#define S_INTENSE (1 << 2)

#define IS_SET(flag, bit) ((flag) & (bit))
#define BIT_SET(var,bit) ((var) |= (bit))
#define BIT_REMOVE(var,bit) ((var) &= ~(bit))

void fnPrintUsage();
void fnPrintVersion();
void fnDeleteFile(char *, int);
void fnPrintStandard(char *str, ...);
void fnUpdateFileTimes(char *file);

char *prog = NULL;
int status = 0;

struct updt_st { 
	int uactime;
	int umodtime;

	struct utimbuf ub;
} updt;

int main(int argc, char **argv)
{
	int c = 0, len = ROUND;
	extern char *optarg;

	srand(time(NULL));

	prog = argv[0];

	updt.uactime = -1;
	updt.umodtime = -1;

	while ((c = getopt(argc, argv, "l:hvVa:m:I")) != EOF)
	{
		switch (c)
		{
			case 'l':
				len = atoi(optarg);

				if ((len <= 0) || (len > 32000))
				{
					fprintf(stdout,"Round is not in a valid range.\n");

					return 0;
				}

				break;
			case 'h':
				fnPrintUsage();
			case 'v':
				fnPrintVersion();
			case 'V':
				BIT_SET(status, S_VERBOSE);
				break;
			case 'a':
				updt.ub.actime = atoi(optarg);
				updt.uactime = 1;
				break;
			case 'm':
				updt.ub.modtime = atoi(optarg);
				updt.umodtime = 1;
				break;
			case 'I':
				BIT_SET(status, S_INTENSE);
				break;
		}
	}

	if (IS_SET(status,S_INTENSE))
		len = 255;

	if (optind < argc)
	{
		while (argv[optind])
		{
			fnDeleteFile(argv[optind], len);
		
			optind++;
		}

	}
	else
		fnPrintUsage();

	return 1;
}	

void fnDeleteFile(char *filename, int len)
{
	FILE *fd;
	char buf[2] = { 0, 0 };
	struct stat statbuf;
	int x = 0;
	
	if (!filename)
		fnPrintUsage();

	if (stat(filename,&statbuf) < 0)
	{
		fnPrintStandard("%s: non-existant file.\n",filename);

		return;
	}

	if (!S_ISREG(statbuf.st_mode))
	{
		fnPrintStandard("%s: refusing to delete non-regular file.\n",filename);

		return;
	}
	
	fnPrintStandard("d: %s s: %lib r: ",filename,statbuf.st_size);
	
	for (x = 0; x < len; x++)
	{
		fd = fopen(filename,"r+");

		if (!fd)
			break;

		while (fread(buf,1,1,fd))
		{
			if (IS_SET(status, S_INTENSE))
				fprintf(fd,"%c",x);
			else
				fprintf(fd,"%c",rand()%255);		
		}

		fnPrintStandard("%i ",x);

		fclose(fd);

		sync();
		
		fnUpdateFileTimes(filename);
	}


	if (unlink(filename) < 0)
		fnPrintStandard(" --  Failure.\n");
	else
		fnPrintStandard(" --  Success.\n");

	return;
}

void fnPrintUsage()
{
	fprintf(stdout,"Usage: %s [OPTION(s)]... [FILE(s)]...\n",prog);
	fprintf(stdout,"Decimate is used to securely delete files from your harddrive.\n\n");
	fprintf(stdout,"\t-l arg\tthe number of rounds to read/write/sync the file. (default=4)\n");
	fprintf(stdout,"\t-I\twrites file 255 times, using every possible char.\n");
	fprintf(stdout,"\t-V\tverbose mode.\n");
	fprintf(stdout,"\t-a arg\tdefine an access time.\n");
	fprintf(stdout,"\t-m arg\tdefine a modification time.\n");
	fprintf(stdout,"\t-h\tthis help menu.\n");
	fprintf(stdout,"\t-v\tversion information.\n\n");
	fprintf(stdout,"!!! WARNING !!! WARNING !!! WARNING !!! WARNING !!! WARNING !!!\n\n");
	fprintf(stdout,"  Files will be unrecoverable in their original format after\n");
	fprintf(stdout,"                  decimate has been run. \n\n");  
	fprintf(stdout,"!!! WARNING !!! WARNING !!! WARNING !!! WARNING !!! WARNING !!!\n");
		
	exit(0);
}

void fnPrintVersion()
{
	fprintf(stdout,"decimate v 1.0.0 by your buddies at www.afro-productions.com\n");

	exit(0);
}

void fnPrintStandard(char *str, ...)
{
	if (IS_SET(status, S_VERBOSE))
	{
		char buf[512];
		va_list ap;
	
		va_start(ap,str);
	
		vsnprintf(buf,512,str,ap);
	
		va_end(ap);

		fprintf(stdout,buf);

		fflush(stdout);
	}
}

void fnUpdateFileTimes(char *file)
{
	struct utimbuf ub;

	if (updt.uactime == -1)
		ub.actime = rand()%time(NULL)+1;
	if (updt.umodtime == -1)
		ub.modtime = rand()%time(NULL)+1;

	utime(file, &ub);

	return;
}
